function [betaOpt, omegaOpt] = FLOG_tuned(X, Y, valX, valY, betaInit, omegaInit, mask, ...
    trnScales, thresholdingWay, tuningMethod, pathLen, errBnd, MB, MO, MA, output )

% This function searches the optimal pair of (lambda, eta) for the FLOG
% algorithm using a '3-path' strategy.
% lambda: penalty parameter for Beta
% eta: penalty parameter for Omega

[n, d] = size(X);
warmStart = 1;
betaInitOri = betaInit; omegaInitOri = omegaInit;

%% searching grid
k0 = norm(X, 2);
X = X ./ k0;
valX = valX./k0;
lamUBndMat = betaInit + (X'*Y - X'*X*betaInit)*omegaInit;
lamUBnd = max(max(abs(lamUBndMat)));
lambdas = lambdaGrid( lamUBnd, 'fusion', pathLen ) ; lambdas(end)=[];  % for sp500:500* 
pathLen = length(lambdas);
maxEta = max(max(abs(cov(Y))));   
lam2ratio = [1/10,1/100,1/200]; 
etas = lam2ratio*maxEta;
pathMinErr = []; 
pathOptLam = [];

%% tuning 
if ~strcmpi(tuningMethod, 'cv')
    for etaPath = 1:length(etas)
        disp(['etaPath#: ',num2str(etaPath)]);
        eta = lam2ratio(etaPath)*maxEta;
        pathErr = []; 
        betaInit = betaInitOri; omegaInit = omegaInitOri; 
        for iPath = 1:pathLen
            lambda = lambdas(iPath);
            if output,   disp(['Solution ', num2str(iPath)]); end
            [ betaEst, omegaEst ] = FLOG( X, Y, thresholdingWay, mask, ...
                errBnd, MB, MO, MA, betaInit, omegaInit, trnScales, lambda, eta, output);
            err = modelSelection( X, Y, valX, valY, betaEst, omegaEst, tuningMethod, 'mrce' );
            pathErr = [pathErr, err]; 
            if warmStart, betaInit = betaEst; omegaInit = omegaEst;  end % path-wise warm start
        end
        [minErr, optInd] = min(pathErr);
        if output, disp(['------ Path ', num2str(etaPath), ': Path length is ', num2str(iPath), '. Optimal Index is ', num2str(optInd), ]); end
        pathMinErr = [pathMinErr, minErr];
        pathOptLam = [pathOptLam, lambdas(optInd)];
    end
end

%% find the optimal pair of lambda and eta
[minErr, optInd] = min(pathMinErr);
optLam = pathOptLam(optInd);
optEta = etas(optInd);
if output, disp(['Optimal lambda on path ', num2str(optInd)]); end
[ betaOpt, omegaOpt ] = FLOG( X, Y, thresholdingWay, mask, ...
    errBnd,  MB, MO, MA, betaInit, omegaInit, trnScales, ...
    optLam, optEta, output);

betaOpt = betaOpt./repmat(trnScales', 1, size(betaInit,2)) / k0;

end

%% FLOG by alternating optimization
function [ betaEst, omegaEst ] = FLOG( X, Y, thWay, mask, errBnd,  MB, MO, MA, betaInit, omegaInit, trnScales, lamB, lamO, output)

% This is the FLOG algorithm. 
% It estimates Beta and Omega using alternative optimization.
% mask: the pattern used for screening
% lamB: penalty parameter for Beta
% lamO: penalty parameter for Omega

[n,d] = size(X);
betaPre =  inv(X'*X+lamB*eye(d))*X'*Y; 
omegaPre = omegaInit;
itrInd = 0;
output = 0;
Bbnd = errBnd*sum(sum(abs(betaPre)));

while 1
    itrInd = itrInd+1;
    
    % Omega-optimization step
    if output,   fprintf('o-'); end
    Sigma = (Y-X*betaPre)'*(Y-X*betaPre)/n;
    omegaEst = graphLASSO([], Sigma, lamO, MO, errBnd, mask, 0); % diagonal elements not penalized
    
    % Beta-optimization step
    if output,   fprintf('b-'); end
    if itrInd==1, betaOld=betaInit; else betaOld=betaPre; end
    betaEst = betaGivenOmega(X, Y, mask, thWay, Bbnd, MB, betaOld, omegaEst, trnScales, lamB, output);
    
    % test convergence
    err = sum(sum(abs(betaPre-betaEst)));  
    if itrInd>=MA
        break;
    end
    if err<Bbnd 
        break;
    end
    
    if output,   if mod(itrInd,20)==0; fprintf('\n');end;  end
    betaPre = betaEst;
    omegaPre = omegaEst;
    
end
if output disp(['Iterations of FLOG = ', num2str(itrInd)]); end

end

