
function [C_hat, betaCur, omegaCur, sigmaCur] = GIST(X, Y, betaCur, omegaCur, trnScales, thWay, lambda, phi, errBnd, maxIT, output )

% This is the function for the GIST algorithm
% X has been scaled. So Beta needs to be scaled back 

%%%%% preprocessing and initialization
XtX = X'*X;
XtY = X'*Y;
[n, d] = size(X);
j = 0;
alpha = 0.01;
fcur = objectiveValue( X, Y, betaCur, omegaCur, n, thWay, lambda, phi );

%%%%% iteration
while 1
    j = j + 1;
    if j>=maxIT
        if output, disp(['no convergence ',  'err = ', num2str(err)]); end
        break;
    end
    
    betaG = (XtX*betaCur-XtY)*omegaCur;
    omegaG = (Y-X*betaCur)'*(Y-X*betaCur)/2-inv(omegaCur)*n/2;
    
    %% armijo search
    alpha = alpha*10;
    while 1
        noupdate = 0;
        % update with step alpha
        [ betaNew, omegaNew, G ] = update( betaCur, omegaCur, betaG, omegaG, alpha, 1, j );
        
        % joint group thresholding
        [ betaNew, omegaNew ] = jointThresholding( betaNew, omegaNew, thWay, lambda, phi);
        
        % validate
        fnew = objectiveValue( X, Y, betaNew, omegaNew, n, thWay, lambda, phi  );
        gap = 1e-4*sum(sum(G.*([betaNew, omegaNew]-[betaCur, omegaCur])));
        if fnew<=fcur+gap && ~(isinf(fnew))
            break;
        elseif alpha<1e-8
            if ~ispd(omegaNew)
                noupdate = 1;
            end
            if (mod(j,2)~=0)
                noupdate = 2;
            end
            break;
        else
            alpha = alpha/10;
        end
    end
    
    if noupdate==1
        continue;
    end
    
    %% check convergence
    err = abs(fcur-fnew)/abs(fcur);
    
    if  err <= errBnd
        if output, disp('Converged'); end
        break;
    end
    if noupdate==2
        break;
    end
    
    if all(all(betaNew-diag(diag(betaNew))==0)) && all(all(omegaNew-diag(diag(omegaNew))==0))
        if output, disp(['all zeros at ', num2str(j)]);  end
        break;
    end
    if isinf(fnew)&&fnew<0
        if output, disp(['objective value reaches -Inf at', num2str(j)]); end
        break;
    end
    if d>=1000 && all(all( (betaNew~=0)==(betaCur~=0) )) && all(all( (omegaNew~=0)==(omegaCur~=0) ))
        if output, disp(['pattern converged at ', num2str(j)]);  end
        break;
    end
    
    %% update the estimate
    if (sum(sum(isnan(betaNew)))~=0) || (sum(sum(isinf(betaNew)))~=0 || sum(sum(isnan(omegaNew)))~=0) || (sum(sum(isinf(omegaNew)))~=0)
        disp('inregular elements!');
    end
    
    betaCur = betaNew;
    omegaCur = omegaNew;
    fcur = fnew;
end

betaCur = betaCur./repmat(trnScales', 1, d); % scale back
betaCur(abs(betaCur)<1e-8) = 0;  % post thresholding
omegaCur(abs(omegaCur)<1e-8) = 0;
sigmaCur = pinv(omegaCur); 
C_hat = sqrt( betaCur.^2+betaCur'.^2 + phi*(omegaCur.^2+omegaCur'.^2) );

end

% compute the value of the objective function
% f = trace((X*Beta-Y)*Omega*(X*Beta-Y)')/2 + n*log(det(Omega))/2 + P(Beta, Omega)
function [ value ] = objectiveValue( X, Y, Beta, Omega, n, thWay, lambda, phi )

traceTerm = trace((X*Beta-Y)*Omega*(X*Beta-Y)')/2;

if ispd(Omega)
    logTerm = n*log(det(Omega))/2;
else
    logTerm = -Inf;
end

value = traceTerm - logTerm;

if ~strcmp(thWay,'quantile')
    penalty = sum(sum(lambda.* sqrt(Beta.^2+phi*Omega.^2)));
else
    penalty =0 ;
end
value = traceTerm - logTerm + penalty;

end

% update based on gradient descent
function [ betaNew, omegaNew, G ] = update( betaCur, omegaCur, betaG, omegaG, alpha, asynchronous, j )
if ~asynchronous
    betaNew = betaCur - alpha * betaG;
    omegaNew = omegaCur - alpha * omegaG;
    G = [betaG, omegaG];
else
    if(mod(j,2)~=0)
        betaNew = betaCur - alpha * betaG;
        omegaNew = omegaCur;
        G = [betaG, zeros(size(omegaG))];
    else
        omegaNew = omegaCur - alpha * omegaG;
        betaNew = betaCur;
        G = [zeros(size(betaG)), omegaG];
    end
end

end

% joint thresholding for C
function [ betaNew, omegaNew ] = jointThresholding( Beta, Omega, thWay, lambda, phi)

d = size(Beta, 1);

switch(lower(thWay))
    case 'quantile'
        normMat = sqrt( Beta.^2+Beta'.^2 + phi*(Omega.^2+Omega'.^2) );
        normMat0 = normMat-diag(diag(normMat));
        sortNorm = sort(normMat0(:), 'descend');
        th = sortNorm(lambda);
        ttInds = normMat0>th;   % normMat>lambdaMat
        betaNew = diag(diag(Beta))/(1+1e-8);
        betaNew(ttInds) =  Beta(ttInds)/(1+1e-8);
        omegaNew = diag(diag(Omega))/(1+1e-8);
        omegaNew(ttInds) = Omega(ttInds)/(1+1e-8);
    case 'hard'
        normMat = sqrt( Beta.^2+Beta'.^2 + phi*(Omega.^2+Omega'.^2) );
        ttInds = normMat>lambda;
        betaNew = zeros(d);
        betaNew(ttInds) =  Beta(ttInds);
        omegaNew = zeros(d);
        omegaNew(ttInds) = Omega(ttInds);
    case 'soft'
        normMat = sqrt( Beta.^2+Beta'.^2 + phi*(Omega.^2+Omega'.^2) );
        ttInds = normMat>lambda;
        betaNew = diag(diag(Beta));          % zeros(d);
        betaNew(ttInds) = Beta(ttInds)./normMat(ttInds).*(normMat(ttInds)-lambda);
        omegaNew = diag(diag(Omega));         % zeros(d);
        omegaNew(ttInds) = sqrt(phi)*Omega(ttInds)./normMat(ttInds).*(normMat(ttInds)-lambda);
end

end
