% ==================================
% Demo for JGSE
% Written by Yuejia He & Yiyuan She
% ==================================

% ==================================
% This is to demostrate the JGSE network learning framework.
% The dynamical network is described by
% x(t) = x(t-1)*Beta + epsilon, where epsilon~N(0,Sigma).
% x(t) is a d-dimensional (row) vector representing node states.
% Beta is a d*d transition matrix;
% Omega = inv(Sigma) is the precision matrix.
% Beta and Omega are supposed to be sparse and share similar structures.
% The joint structure of Beta and Omega are captured by matrix C?
% C^2 = Beta.^2+phi^2*Omega.^2.
% Given observations of the network: x(1),...,x(n).
% Let X=[x(1),...,x(n-1)], Y=[x(2),...,x(n)].
% The negative log-likelihood function of (Beta, Omega) is
% L(Beta, Omega)=1/2*tr((X*Beta-Y)*Omega*(X*Beta-Y)^T)-n/2*log|Omega|
% The JGSE network learning framework consists of two steps:
% The first step applies the GIST algorithm to estimate matrix C for JAG
% screening and robust decomposition:
% min L(Beta, Omega), s.t. ||C||_0<=qd^2.
% The second step applies the FLOG algorithm to each subnetwork and obtain
% fine estimates for Beta and Omega:
% min L(Beta, Omega)+P(Beta; lambda)+P(Omega; eta),
% s.t. E(Beta) in E(C) and E(Omega) in E(C).
% Refer to the paper for more details.
% ==============================


clear; clc;
randn('state', 0); rand('state', 0);
output = 1;

%% simulate the network
d = 40;               % network size
ds=[20,20];           % a vector of subnetwork sizes. vector sum must equal to d
K = length(ds);       % number of subnetworks
[ betaTrue, sigmaTrue, omegaTrue, jointTrue, labels ] = generateModel( d, ds, 0.2, 1 );
  
if output
    figure;
    subplot(2,2,1); imagesc(abs(betaTrue));  title('True B'); colormap('gray');
    subplot(2,2,2); imagesc(abs(omegaTrue)); title('True Omega'); colormap('gray');
    subplot(2,2,3); imagesc(abs(jointTrue)); title('True C'); colormap('gray');
end

%% generate and standardize data
n = 200;                                 % number of training data
valN = 1000;                             % number of validation data
tstN = 0;                                % number of testing data
centerX = 1; scaleX = 1; centerY = 1;    % data standardization.
[X, Y, valX, valY, tstX, tstY, noise] = timeSeries( betaTrue, sigmaTrue, d, n, valN, tstN);  % Generate Data: generate mAR1 time-series data
[X_std, trnCenters, trnScales, Y_std, trnCenterY, valX_std, tstX_std, valY_std, tstY_std] = dataStand(centerX, scaleX, X, centerY, Y, valX, tstX, valY, tstY);
if output
    figure;
    subplot(2,1,1); plot(X); title('X');
    subplot(2,1,2); plot(X_std); title('X standardized');
end


%% run GIST
tic
disp('Running GIST')
% apply GIST to obtain C_hat
q = 0.3;                                 % quantile parameter
betaInit = zeros(d); omegaInit = eye(d); % initial values
phi = 1;                                 % penalty weight on Omega. set to 1 given no prior
maxIT = 1e3;                             % maximum iteration number
errBnd = 1e-3;                           % error tolerance
C_hat = GIST(X_std, Y_std, betaInit, omegaInit, trnScales, 'quantile', q*(d*d-d), phi, errBnd, maxIT, output );

% robust decomposition based on C_hat
clusters = clustering(C_hat, K);

% evaluate the decomposition results using Rand index (RI):
% RI=?TP+TF)/(TP+TF+FP+FN)
% TP: a pair coming from the same cluster are assigned to a single cluster
% TF: a pair coming from different clusters are assigned to different clusters
% FP: a pair coming from different clusters are assigned to the same cluster
% FN: a pair coming from the same cluster are assigned to different clusters
RI = evaluateCluster(clusters, labels);
disp(['Rand Index: ', num2str(RI)]);
toc

%% run FLOG
tic
disp(' '); disp('Running FLOG')
thWay = 'soft';                          % thresholding rule
tuningMethod = 'validation';             % model selection criterion: bic, aic, validation
pathLen = 10;                            % number of solutions on a path
MB = 100;                                % maximum iteration number for the B-optimization step
MO = 100;                                % maximum iteration number for the Omega-optimization step
MA = 100;                                % maximum iteration number for the outer iteration
clusInds = unique(clusters); betaOpt = zeros(d); omegaOpt = zeros(d);
% apply FLOG to each subnetwork
for subInd = 1:length(clusInds)
    disp(['Subnetwork ', num2str(subInd)])
    thisSub = find(clusters==clusInds(subInd)); thisSize = length(thisSub); thisScales = trnScales(thisSub);
    thisX = X_std(:,thisSub); thisY = Y_std(:,thisSub);thisValX = valX_std(:,thisSub); thisValY = valY_std(:,thisSub);
    thisBetaInit = zeros(thisSize); thisOmegaInit = eye(thisSize);
    thisMask = C_hat(thisSub, thisSub); thisMask = thisMask~=0;  % apply C_hat for screening
    [thisBetaOpt, thisOmegaOpt] = FLOG_tuned(thisX, thisY,thisValX,thisValY, thisBetaInit, thisOmegaInit, ones(thisSize), ...
        thisScales, thWay,  tuningMethod, pathLen, errBnd, MB, MO, MA, output );
    betaOpt(thisSub, thisSub) = thisBetaOpt;  omegaOpt(thisSub, thisSub) = thisOmegaOpt;
end

% evaluate the estimation and identification accuracy
[TPR, FPR, ME] = performanceEvaluate(betaTrue, omegaTrue, betaOpt, omegaOpt, X_std);
disp(['True Positive Rate  = ', num2str(TPR)])
disp(['False Positive Rate = ', num2str(FPR)])
disp(['     Model Error    = ', num2str(ME)])
toc

