% ==============================
% A matlab version for graphical LASSO 
% written by Huanghuang
% modified by Yuejia
% ==============================
% Input:  S, sample covariance matrix, p by p
%         lam, sparsity tuning parameter
% Output: invSigmaEst, the estimate of inv(Sigma)
%          where Sigma = arg max_{X semi positive definite}
%                        logdet(X)-trace(SX)-P(X,lam)

% ==============================
function invSigmaEst = graphLASSO(W0, S,lam, maxIter, thr, mask, diagPen)
errs = [];
% ==============================
% Parameter needed
p = size(S,1);

% ==============================
if isempty(W0)
    W = S+lam*eye(p); % cold start
else
    W = W0;
end

for iter = 1:maxIter
%     disp(num2str(iter));
    oldW = W;
    for i = 1:p
        W11 = W;
        W11(i,:) = []; W11(:,i) = [];
        w12 = W(:,i); w12(i) = [];
        s12 = S(:,i); s12(i) = [];
        lamVec = lam*ones(p-1,1); maskVec = mask(:,i); maskVec(i) = [];
        beta = vectorLASSO(W11,s12,w12,lamVec,thr, maskVec);  %thr/sum(sum(abs(W11)))
        w = W11*beta;
        
        W(1:i-1,i) = w(1:i-1);
        W(i+1:end,i) = w(i:end);
        W(i,1:i-1) = w(1:i-1)';
        W(i,i+1:end) = w(i:end)';
    end
    err = mean(mean(abs(oldW-W))); errs=[errs, err];  % max(sum(abs(oldW-W),1));
    if err<thr
%         disp(['graph LASSO converge at iteration ' num2str(iter)]);
        break;
    end
end
% if iter>=maxIter
%     disp(['graph LASSO NOT converge']);
% end
    
if diagPen==0
    for i = 1:p
        W(i,i) = S(i,i);
    end
end

invSigmaEst = inv(W); % not efficient, can be replaced by Friedman's code
invSigmaEst(abs(invSigmaEst)<thr) = 0;

end


% ==============================
% A vector LASSO update

% ==============================
% Input:  W11, p-1 by p-1
%         s12, p-1 vector
%         lam, sparsity tuning parameter
% Output: w12, the estimate of inv(Sigma)
%          where Sigma = arg max_{X semi positive definite}
%                        logdet(X)-trace(SX)-P(X,lam)

% ==============================
function w = vectorLASSO(W11,s12,w12,lam,thres, mask)

% ==============================
% Parameters needed
maxIter = 2000;

% ==============================
p = size(W11,1);
w = w12;
% iteration
for iter = 1:maxIter
    oldW = w;
    for i = 1:p
        t = s12(i)-W11(:,i)'*w+W11(i,i)*w(i);
        if mask(i)==0 || abs(t)<lam(i)
            w(i) = 0;
        else
            w(i) = sign(t)*(abs(t)-lam(i))/W11(i,i); % LASSO
        end
    end
    if max(abs(w-oldW))<thres
       % fprintf([num2str(iter),',']);
        break;
    end
end
% if iter>=maxIter
%     fprintf('inner iteration does not converge');
% end

end