
function [ risk, negloglik, modelcomp ] = modelSelection( X, Y, valX, valY, betaEst, omegaEst, tuningMethod, learningMethod )

% This function calculates the 'risk' of an estimated model
% Available criteria include bic and aic
% Learning methods include
% 'joint': estimate C
% 'mrce': jointly estimate Beta and Omega
% 'beta': estimate Beta only
% 'ggm': estimate Omega only

[n,d] = size(Y);

%% number of parameters (degree of freedom)
switch(lower(learningMethod))
    case{'joint'}
        freeK = nnz(sqrt( betaEst.^2+betaEst'.^2 + (omegaEst.^2+omegaEst'.^2) ));
    case{'mrce'}
        freeK = nnz(betaEst) + nnz(omegaEst)/2;
    case{'beta'}
        freeK = nnz(betaEst);
    case{'ggm'}
        freeK = nnz(omegaEst)/2;
end

%% risk
switch(lower(tuningMethod))
    case{'validation'}
        errTmp = valX*betaEst-valY;
        negloglik = trace(errTmp*omegaEst*errTmp')-log(det(omegaEst))*size(valX,1);
%         if (strcmpi(evalCriterion, 'fitting')), negloglik = trace(errTmp*errTmp'); end
        modelcomp = 0;
        risk = negloglik + modelcomp;
        
    case{'bic'}  % BIC = K*ln(n) - 2ln(L)
        errTmp = X*betaEst-Y;
        negloglik = trace(errTmp*omegaEst*errTmp')-log(det(omegaEst))*n; % accually it is 2*negloglik
        modelcomp = freeK*log(n);
        risk = negloglik + modelcomp;     
    case('aic')   % AIC = 2K - 2ln(L)
        errTmp = X*betaEst-Y;
        negloglik = trace(errTmp*omegaEst*errTmp')-log(det(omegaEst))*n;
        modelcomp = freeK*2;
        risk = negloglik + modelcomp;
    case('ebic')
        disp('implementation coming')
end

%% reject naive solution
switch(lower(learningMethod))
    case{'joint'}
        if all(all(betaEst-diag(diag(betaEst))<1e-8)) && all(all(omegaEst-diag(diag(omegaEst))<1e-8))
            risk = Inf;
        end        
    case{'mrce'}
        if all(all(betaEst-diag(diag(betaEst))<1e-8)) && all(all(omegaEst-diag(diag(omegaEst))<1e-8))
            risk = Inf;
        end       
    case{'beta'}
        if all(all(betaEst-diag(diag(betaEst))<1e-8))
            risk = Inf;
        end
    case{'ggm'}
        if all(all(omegaEst-diag(diag(omegaEst))<1e-8))
            risk = Inf;
        end
end

end


